# -*- coding: utf-8 -*-
"""
Created on Sat Nov 14 16:31:46 2020

@author: Korean_Crimson
"""

import datetime

def generate_timestamp() -> str:
    '''Generates date+timestamp of current time'''
    return datetime.datetime.now().strftime('%Y%m%d_%H%M%S')

def get_current_date() -> str:
    '''Returns current date'''
    return datetime.datetime.now().strftime('%Y-%m-%d')

def validate_date_string(date_string) -> bool:
    '''Checks if passed string can be converted to date'''
    try:
        datetime.datetime.strptime(date_string, '%Y-%m-%d')
        valid = True
    except ValueError:
        valid = False
    return valid



# -*- coding: utf-8 -*-
"""
Created on Sat Dec  4 16:47:28 2021

@author: richa
"""
from typing import Dict
from typing import Type

from desktop_shop.gui import components

from . import checkout
from . import home
from . import login
from . import main_menu
from . import profile
from . import register


class View(components.View):
    """View Protocol interface"""

    @classmethod
    def create(cls, window: components.Tk, builder: components.Builder) -> components.View:
        """Creates the View"""

views: Dict[str, Type[View]] = {
    "checkout": checkout.View,
    "home": home.View,
    "login": login.View,
    "main_menu": main_menu.View,
    "profile": profile.View,
    "register": register.View,
}



# -*- coding: utf-8 -*-
"""
Created on Sat Dec  4 16:43:01 2021

@author: richa
"""
import tkinter as tk

from desktop_shop import user
from desktop_shop import util
from desktop_shop.gui import callbacks
from desktop_shop.gui import components

class View(components.View):
    """Register view"""

    @classmethod
    def create(cls, window: components.Tk, builder: components.Builder):
        '''Initialises register view, including all its components'''
        builder.view = cls()
        builder.root = window

        frame = builder.create('frame', name='register', relief=tk.SUNKEN, bd=2)
        frame.place(row=8, row_span=7, col=1, col_span=3, sticky='nsew')
        builder.root = frame.component.tk_component

        builder.create('label', text="Register").place(row=0, col=1)

        #email
        builder.create('label', text="*Email address").place(row=1, col=0)
        entry = builder.create('entry', name='email', textvariable=tk.StringVar())
        entry.place(row=1, col=1, sticky='nsew')

        #first name
        builder.create('label', text="*First name").place(row=2, col=0)
        entry = builder.create('entry', name='first_name', textvariable=tk.StringVar())
        entry.place(row=2, col=1, sticky='nsew')

        #last name
        builder.create('label', text="*Last name").place(row=3, col=0)
        entry = builder.create('entry', name='last_name', textvariable=tk.StringVar())
        entry.place(row=3, col=1, sticky='nsew')

        #gender
        builder.create('label', text="Gender").place(row=4, col=0)
        entry = builder.create('entry', name='gender', textvariable=tk.StringVar())
        entry.place(row=4, col=1, sticky='nsew')

        #date of birth
        builder.create('label', text="Date of Birth").place(row=5, col=0)
        entry = builder.create('entry', name='dob', textvariable=tk.StringVar())
        entry.place(row=5, col=1, sticky='nsew')

        #password
        builder.create('label', text="*Password").place(row=6, col=0)
        entry = builder.create('entry', name='pw', textvariable=tk.StringVar(), show='*')
        entry.place(row=6, col=1, sticky='nsew')

        #confirm pw
        builder.create('label', text="*Confirm Password").place(row=7, col=0)
        entry = builder.create('entry', name='confirm_pw', textvariable=tk.StringVar(), show='*')
        entry.place(row=7, col=1, sticky='nsew')

        builder.create('label', text="*required").place(row=8, col=1)
        builder.create('button2', text='Register', command=callbacks.register).place(row=9, col=1)
        return builder.view

    def clear_entries(self):
        """Clears all entries in the register view"""
        for entry in self.filter('entry'):
            entry.set_var('')

    def get_user_data(self):
        """Returns a UserSignUpData object containing data from register view entries"""
        entry_names = ['first_name', 'last_name', 'gender', 'dob', 'email']
        user_data = user.UserSignUpData(*(self[f'{name}_entry'].get_var() for name in entry_names))
        user_data.join_date = util.get_current_date()
        return user_data



# -*- coding: utf-8 -*-
"""
Created on Sat Dec  4 17:02:53 2021

@author: richa
"""
import functools
import tkinter as tk
from typing import List

from desktop_shop import user
from desktop_shop.gui import callbacks
from desktop_shop.gui import components

#pylint: disable=line-too-long
class View(components.View):
    """Profile view"""

    @classmethod
    def create(cls, window: components.Tk, builder: components.Builder):
        '''Initialises profile view, including all its components'''
        builder.view = cls()
        builder.root = window

        frame = builder.create('frame', relief=tk.RAISED, bd=3)
        frame.place(row=2, row_span=1, col=1, col_span=3, sticky='nsew')
        builder.root = frame.component.tk_component

        #date joined
        builder.create('label', text="Date joined:").place(row=0, col=0)
        label = builder.create('label', name='date_joined_data', textvariable=tk.StringVar())
        label.place(row=0, col=1, sticky='w')

        #profile data frame
        builder.root = window
        frame = builder.create('frame', relief=tk.RAISED, bd=3)
        frame.place(row=3, row_span=6, col=1, col_span=3, sticky='new')
        builder.root = frame.component.tk_component

        #profile data
        _create_profile_data_entry(builder, text="First name:", name='first_name', row=0)
        _create_profile_data_entry(builder, text="Last name:", name='last_name', row=1)
        _create_profile_data_entry(builder, text="Gender:", name='gender', row=2)
        _create_profile_data_entry(builder, text="Email address:", name='email', row=3)
        _create_profile_data_entry(builder, text="Date of Birth:", name='dob', row=4)
        button = builder.create('button2', name='edit_user_data',
                                text='Confirm changes', command=callbacks.edit_user_data, bd=3)
        button.place(row=5, col=1, col_span=2, sticky='nse')


        #password change
        builder.root = window
        frame = builder.create('frame', relief=tk.RAISED, bd=3)
        frame.place(row=10, col=1, col_span=3, sticky='nsew')
        builder.root = frame.component.tk_component

        button = builder.create('button', text='Change password', name='password_change_frame',
                                command=callbacks.show_password_change_frame, bd=3)
        button.place(row=0, col=2, sticky='w')

        frame = builder.create('frame', name='password_change', relief=tk.RAISED, bd=0)
        frame.place(row=0, row_span=3, col=0, col_span=3)
        builder.root = frame.component.tk_component
        builder.view.hide_components('password_change_frame')

        #password
        builder.create('label', text='*Password').place(row=0, col=0)
        builder.create('entry', name='pw', textvariable=tk.StringVar(), show="*").place(row=0, col=1, sticky='we')

        #confirm password
        builder.create('label', text='*Confirm').place(row=1, col=0)
        builder.create('entry', name='confirm_pw', textvariable=tk.StringVar(), show="*").place(row=1, col=1, sticky='we')

        button = builder.create('button2', text='Confirm password change', command=callbacks.edit_user_password, bd=3)
        button.place(row=2, col=1, sticky='e')

        return builder.view

    def store_user_data(self, user_data):
        """Stores the passed user data in the user data entries"""
        for name, value in zip(self.user_data_entries, user_data):
            self[name].set_var(value)

    def get_user_data(self):
        """Returns a UserSignUpData object containing the data in the entries"""
        return user.UserSignUpData(*(self[e].get_var() for e in self.user_data_entries))

    @property
    def user_data_entries(self) -> List[str]:
        """Getter for user_data_entries, returns list of data entries in the profile view"""
        entry_names = ['first_name', 'last_name', 'gender', 'dob', 'email']
        return [f'{name}_entry' for name in entry_names] + ['date_joined_data_label']


def _create_profile_data_entry(
    builder: components.Builder,
    text: str,
    name: str,
    row: int
    ) -> None:
    """Creates a labelled entry + edit button"""
    builder.create('label', text=text).place(row=row, col=0, sticky='w')

    entry = builder.create('entry', name=name, textvariable=tk.StringVar())
    entry.place(row=row, col=1, sticky='we')
    entry.component.config(state='disabled')

    callback = functools.partial(callbacks.activate_profile_entry, entry.component.name)
    button = builder.create('button', text='Edit', command=callback, bd=3)
    button.place(row=row, col=2, sticky='e')



# -*- coding: utf-8 -*-
"""
Created on Sat Dec  4 17:04:48 2021

@author: richa
"""
import tkinter as tk

from desktop_shop.gui import callbacks
from desktop_shop.gui import components

class View(components.View):
    """Login view"""

    @classmethod
    def create(cls, window: components.Tk, builder: components.Builder):
        '''Initialises login view, including all its components'''
        builder.view = cls()
        builder.root = window

        #login frame
        frame = builder.create('frame', relief=tk.SUNKEN, bd=2)
        frame.place(row=8, row_span=4, col=1, col_span=3, sticky='nsew')
        builder.root = frame.component.tk_component

        builder.create('label', text='Login').place(row=0, col=1)

        #email
        builder.create('label', text='Email address').place(row=1, col=0)
        edit = builder.create('entry', name='email', textvariable=tk.StringVar())
        edit.place(row=1, col=1, sticky='nsew')

        #password
        builder.create('label', text='Password').place(row=2, col=0)
        edit = builder.create('entry', name='pw', textvariable=tk.StringVar(), show="*")
        edit.place(row=2, col=1, sticky='nsew')

        builder.create('button2', text='Log in', command=callbacks.login).place(row=3, col=1)

        #login unsuccessful
        message = 'Logging in has failed. Please check your credentials.'
        label = builder.create('label2', name='login_failed', text=message)
        label.place(row=4, col=0, col_span=3)
        builder.view.hide_components('login_failed_label')

        return builder.view

    def clear_entries(self):
        """Clears the password and email entries"""
        for entry in ['pw_entry', 'email_entry']:
            self[entry].set_var('')



# -*- coding: utf-8 -*-
"""
Created on Mon Feb  1 10:01:35 2021

@author: Korean_Crimson
"""

TITLE = 'Offline Shop'

#colours
FG = '#FFFFFF'
DISABLED_FG = '#EEEEEE'
BG = '#121212'
BG2 = '#242424'
BG3 = '#363636'
BG4 = '#484848'
BG5 = '#606060'
PRIM = '#3700B3'
SEC = '#A172E1'
ERR = '#CF6679'

#themes
ENTRY_THEME = {'fg': FG, 'bg': BG4, 'disabledbackground': BG5, 'disabledforeground': DISABLED_FG}
FRAME_THEME = {'bg': BG2, 'highlightbackground': BG3}
THEME = {**FRAME_THEME, 'fg': FG}

LABEL_THEME = {'fg': FG, 'bg': BG2, 'highlightbackground': BG2}
BUTTON_THEME = {'fg': FG, 'bg': BG3, 'activebackground': BG4, 'activeforeground': FG}
BUTTON_THEME2 = {'fg': FG, 'bg': PRIM, 'activebackground': BG4,
                 'activeforeground': FG, 'disabledforeground': DISABLED_FG}
DROPDOWN_THEME = {'fg': FG, 'bg': BG2, 'activebackground': BG2, 'activeforeground': FG}
ERROR_THEME = {'fg': ERR, 'bg': BG2}



# -*- coding: utf-8 -*-
"""
Created on Mon Feb  1 09:53:11 2021

@author: Korean_Crimson
"""
from __future__ import annotations

import os
import re
import sys
import tkinter as tk
import uuid
from dataclasses import dataclass
from dataclasses import field
from typing import Any
from typing import Callable
from typing import Dict
from typing import List
from typing import Optional
from typing import Type
from typing import Union

Root = Union[tk.Tk, tk.Widget]


class Tk(tk.Tk):
    """Wrapper around tk.Tk class with easier rowconfigure and columnconfigure functionality"""

    def __init__(self):
        super().__init__()
        self.col_num: int = 0
        self.row_num: int = 0

    def add_row(self, minsize: int, weight: int = 1) -> None:
        """Wraps around tk.Tk.rowconfigure"""
        self.rowconfigure(self.row_num, minsize=minsize, weight=weight)
        self.row_num += 1

    def add_col(self, minsize: int, weight: int = 1) -> None:
        """Wraps around tk.Tk.columnconfigure"""
        self.columnconfigure(self.col_num, minsize=minsize, weight=weight)
        self.col_num += 1


@dataclass
class Gui:
    """Class to contain all tk widgets, tk Vars and other data related to the gui"""

    window: Tk
    views_dict: Dict[str, View] = field(default_factory=dict)
    data: Dict[str, Any] = field(default_factory=dict)

    def __post_init__(self):
        self.builder: Optional[Builder] = None

    def __enter__(self):
        self.pack_all()
        return self

    def __exit__(self, *_):
        pass

    def __getitem__(self, key: str) -> Optional[View]:
        return self.views_dict.get(key)

    def mainloop(self) -> None:
        """Wraps around tk.Tk.mainloop"""
        self.window.mainloop()

    def pack_all(self) -> None:
        """Places all active views on the gui by calling pack"""
        for view in self.views_dict.values():
            if view.active:
                view.pack()
            else:
                view.unpack()

    def deactivate_all(self) -> None:
        """Deactivates all views"""
        for view in self.views_dict.values():
            view.deactivate()

    def switch_to(self, *keys: str) -> None:
        """Switches to a particular view (deactivates all other views)"""
        self.deactivate_all()
        for key in keys:
            self.views_dict[key].activate()
        self.pack_all()


class View:
    """View class that acts as a container of a number of related Component objects"""

    def __init__(self):
        self.active = False
        self._components: Dict[str, Component] = {}
        self._entries = {}
        self._frame_components: Dict[str, Frame] = {}

    def __getitem__(self, component_name):
        """Returns the component identified by the passed name"""
        return self._all_components[component_name]

    def get_frames(self) -> List[Frame]:
        """Returns all Components that are marked as frames"""
        return self._frame_components.values()

    def clear(self) -> None:
        """Clears the current view by unpacking all components and clearing all
        component dicts (uninitialises the entire view).
        """
        self.unpack()
        self._components = {}
        self._frame_components = {}

    def filter(self, value: str) -> List[Component]:
        """Returns components containing the passed string in their name"""
        return [
            c
            for c in self._all_components.values()
            if c.name is not None and value in c.name
        ]

    def activate(self) -> None:
        """Activates the current view (when the Gui object next calls pack on it,
        this View will show when active)
        """
        self.active = True

    def deactivate(self) -> None:
        """Deactivates the current view (when the Gui object next calls pack on it,
        this View will not show when deactive)
        """
        self.active = False

    def add_component(self, component: Component, name: str) -> None:
        """Adds the passed Component under the specified name and stores it in the view"""
        self._components[name] = component

    def add_entry_component(self, component: Component, name: str) -> None:
        """Adds the passed entry component under the specified name and stores it in the view"""
        self._components[f"{name}_entry"] = component
        self._entries[name] = component

    def add_frame_component(self, component: Component, name: str) -> None:
        """Adds the passed Component under the specified name and stores it in the view, as
        a component of type frame
        """
        self._frame_components[name] = component

    def pack(self) -> None:
        """Calls gridpack on all components in the view (frames first)"""
        for component in self._all_components.values():
            component.gridpack()

    def repack(self) -> None:
        """Unpacks, then repacks the view"""
        self.unpack()
        self.pack()

    def unpack(self) -> None:
        """Calls unpack for all components in the view (frames first)"""
        for component in self._all_components.values():
            component.unpack()

    def hide_components(self, *component_names: str) -> None:
        """Hides a particular component"""
        for component_name in component_names:
            self._all_components[component_name].hide()

    def unhide_components(self, *component_names: str) -> None:
        """Unhides a particular component"""
        for component_name in component_names:
            component = self._all_components[component_name]
            component.unhide()
            component.gridpack()

    @property
    def _all_components(self) -> Dict[str, Component]:
        return {**self._components, **self._frame_components}


# pylint: disable=too-many-instance-attributes
@dataclass
class Component:
    """Wrapper class around tk widgets, that stores all data usually required when
    calling .grid(). Using this class makes it possible to repeatedly call grid()
    and grid_forget() for variable gui layouts without passing the arguments repeatedly
    """

    tk_component: tk.Widget
    row: int = 0
    col: int = 0
    sticky: str = "n"
    padx: int = 0
    pady: int = 0
    col_span: int = 1
    row_span: int = 1
    var: Any = None
    name: str = ""

    def __post_init__(self):
        self.data = None
        self.hidden = False

    def hide(self) -> None:
        """Hides the component and calls unpack"""
        self.hidden = True
        self.unpack()

    def unhide(self) -> None:
        """Unhides the component"""
        self.hidden = False

    def add_to(self, view: View) -> None:
        """Adds the component to the passed view"""
        view.add_component(self, self.name)

    def gridpack(self) -> None:
        """calls .grid on the tk_component if it is not hidden"""
        if self.hidden:
            return

        self.tk_component.grid(
            row=self.row,
            column=self.col,
            sticky=self.sticky,
            padx=self.padx,
            pady=self.padx,
            rowspan=self.row_span,
            columnspan=self.col_span,
        )

    def unpack(self) -> None:
        """calls .grid_forget on the tk_component"""
        self.tk_component.grid_forget()

    def get(self) -> Any:
        """calls .get on the tk_component"""
        return self.tk_component.get() # pylint: disable=no-member

    def get_var(self) -> Any:
        """calls .get on the stored tk Var"""
        return self.var.get() if self.var else None

    def set_var(self, value) -> None:
        """calls .set on the stored tk Var"""
        if self.var:
            self.var.set(value)

    def config(self, *args, **kwargs) -> None:
        """calls .config on the tk_component"""
        self.tk_component.config(*args, **kwargs)


class Frame(Component):
    """Frame compoenent"""

    def add_to(self, view: View) -> None:
        """Adds the frame component to the view. Overrides Component.add_to"""
        view.add_frame_component(self, self.name)


@dataclass
class ComponentPlacer:
    """Component placer class, returned from Factory.create"""

    component: Component

    def place(self, **kwargs: Any):
        """Places the component using the specified keyword arguments"""
        for key, value in kwargs.items():
            setattr(self.component, key, value)


@dataclass
class Factory:
    """Factory object, contains a method and a respective component class,
    as well as creation arguments and keyword arguments.
    """

    method: Callable[..., tk.Widget]
    component: Type
    args: List[Any] = field(default_factory=list)
    kwargs: Dict[str, Any] = field(default_factory=dict)

    def create(
        self, root: Root, name: str = None, **kwargs: Any
    ) -> ComponentPlacer:
        """Creates a new Component by calling the creation method. Returns a ComponentPlacer"""
        widget = self.method(root, *self.args, **self.kwargs, **kwargs)
        component = self.component(widget, name=name)
        return ComponentPlacer(component)


# pylint: disable=too-few-public-methods
class EntryFactory(Factory):
    """Entry factory"""

    def create(
        self, root: Root, name: str = None, **kwargs: Any
    ) -> ComponentPlacer:
        """Overrides Factory.create.
        Adds an extra textvariable to the widget creation process.
        """
        component_placer = super().create(root, name, **kwargs)
        component_placer.component.var = kwargs.get("textvariable")
        return component_placer


@dataclass
class Builder:
    """Component builder class"""

    factory_methods: Dict[str, Factory] = field(default_factory=dict)

    def __post_init__(self):
        self.view: View = None
        self.root: Root = None

    def register(self, method_name: str, factory: Factory) -> None:
        """Register a factory under a specified name"""
        self.factory_methods[method_name] = factory

    def create(
        self, method_name: str, name: str = None, **kwargs: Any
    ) -> ComponentPlacer:
        """Creates a component (widget) by looking up the passed method_name
        from the registered factories and calling create on it.
        Passes received keyword arguments to the factory create method.
        Returns the component's ComponentPlacer.
        """
        factory = self.factory_methods[method_name]

        method_name_ = re.sub("[0-9]+", "", method_name)
        name = f"{name}_{method_name_}" if name is not None else str(uuid.uuid4())

        component_placer = factory.create(self.root, name, **kwargs)
        component_placer.component.add_to(self.view)
        return component_placer



# -*- coding: utf-8 -*-
"""
Created on Sat Dec  4 16:57:41 2021

@author: richa
"""
import tkinter as tk

from desktop_shop.gui import callbacks
from desktop_shop.gui import components
from desktop_shop.gui import config

#pylint: disable=line-too-long
class View(components.View):
    """Main menu view"""

    @classmethod
    def create(cls, window: components.Tk, builder: components.Builder):
        '''Initialises main menu view, including all its components'''
        builder.view = cls()
        builder.root = window

        frame = builder.create('menu_frame')
        frame.place(row=0, col=0, col_span=5, row_span=1, sticky='nsew')
        builder.root = frame.component.tk_component

        #buttons
        builder.create('button', name='home', command=callbacks.switch_to_home, text='Home').place(row=0, col=0)
        builder.create('button', name='register', command=callbacks.switch_to_register, text='Register').place(row=0, col=1)
        builder.create('button', name='login', command=callbacks.switch_to_login, text='Login').place(row=0, col=2, sticky='nsew')

        button = builder.create('button', name='checkout', text='Checkout', command=callbacks.switch_to_checkout)
        button.place(row=0, col=3)
        builder.view.hide_components('checkout_button')

        #logged in as frame
        logged_in_as_frame = tk.Frame(window, borderwidth=0, highlightthickness=0, bg=config.BG2)
        component = components.Component(logged_in_as_frame, row=0, col=4, sticky='e')
        builder.view.add_frame_component(component, 'logged_in_as_frame')
        builder.view.hide_components('logged_in_as_frame')

        #options dropdown
        variable = tk.StringVar()
        options = ['Profile', 'Sign out']
        variable.set('Logged in as {}')
        variable.trace('w', callbacks.on_dropdown_value_write_event)
        options_dropdown = tk.OptionMenu(logged_in_as_frame, variable, *options)
        options_dropdown['menu'].config(**config.DROPDOWN_THEME)
        component = components.Component(options_dropdown, row=0, col=7, col_span=2, sticky='nsew', var=variable)
        component.config(**config.DROPDOWN_THEME)
        component.config(highlightthickness=0, bd=3)
        builder.view.add_component(component, 'options_dropdown')

        #message frame
        builder.root = window
        frame = builder.create('frame', name='message', relief=tk.RIDGE, bd=0)
        frame.place(row=0, col=3, col_span=1, sticky='w')
        builder.view.hide_components('message_frame')
        builder.root = frame.component.tk_component

        builder.create('label', name='message', textvariable=tk.StringVar()).place(row=0, col=0)
        builder.create('label2', name='error_message', textvariable=tk.StringVar()).place(row=0, col=0)

        return builder.view
